CraftBarData = {};

-- Bindings
BINDING_HEADER_CRAFTBARHEADER = "CraftBar";
BINDING_NAME_CRAFTBARSNAP = "MouseSnap";

local LBF = LibStub("LibButtonFacade", true);
local buttongroup = nil;

local initialized = false;

local craftbar_professions = {
	['01Tailoring'] = 3908,
	['02Enchanting'] = 7411,
	['03Disenchant'] = 13262,
	['04Smelting'] = 2656,
	['05Blacksmithing'] = 3100,
	['06Alchemy'] = 2259,
	['07Leatherworking'] = 2108,
	['08Engineering'] = 4036,
	['09Jewelcrafting'] = 25229,
	['10Inscription'] = 45357,
	['11Runeforging'] = 53428,
	['12Survey'] = 77364,
	['13Archaeology'] = 78670,
	['14Cooking'] = 2550,
	['15First Aid'] = 3273,
}

local craftbar_buttons = {
	['01Tailoring'] = nil,
	['02Enchanting'] = nil,
	['03Disenchant'] = nil,
	['04Smelting'] = nil,
	['05Blacksmithing'] = nil,
	['06Alchemy'] = nil,
	['07Leatherworking'] = nil,
	['08Engineering'] = nil,
	['09Jewelcrafting'] = nil,
	['10Inscription'] = nil,
	['11Runeforging'] = nil,
	['12Survey'] = nil,
	['13Archaeology'] = nil,
	['14Cooking'] = nil,
	['15First Aid'] = nil,
}

-- functions
function CraftBar_OnLoad(self)

 	DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's |rCraftBar loaded.", 1.0, 1.0, 1.0, 1, 10);

 	self:RegisterEvent("ADDON_LOADED");

	self:RegisterEvent("PLAYER_ENTER_COMBAT");
	self:RegisterEvent("PLAYER_LEAVE_COMBAT");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");

	SLASH_RAIDICONBAR1 = "/craftbar";
  	SLASH_RAIDICONBAR2 = "/craft";
  	SlashCmdList["CRAFTBAR"] = function(msg)
		CraftBar_SlashCommandHandler(msg);
	end
end

function CraftBar_OnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
	
		local addonName = ...;
		if (addonName == "CraftBar") then	
		    if (LBF) then
                buttongroup = LBF:Group("CraftBar");
            end
		end
	end
	
	if (event == "PLAYER_ENTER_COMBAT" or event == "PLAYER_REGEN_DISABLED") then
	    local frame = getglobal("CraftBarFrame");
	    frame:Hide();
	end
end

function CraftBar_SlashCommandHandler(msg)
	if (msg == "reload") then
	 	ReloadUI();
	end
end

function CraftBar_OnUpdate(self, elapsed)
    CraftBar_UpdateCooldowns(self);
end

function CraftBar_UpdateCooldowns(self)
		
	local cooldown = _G[self:GetName().."Cooldown"];
	
    local item = self:GetAttribute("item");
    local spell = self:GetAttribute("spell");
    
    local start, duration, enable = GetSpellCooldown(spell);
  	
    if (cooldown and start and duration and enable) then
        CooldownFrame_SetTimer(cooldown, start, duration, enable);
    end
end

function CraftBar_initButtons()
 	
    CraftBarFrame:SetBackdrop( { 
        --bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    
    CraftBarFrame:Hide();

	for name, spellid in pairs(craftbar_professions) do 	    
 	    local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	
        local button = CreateFrame("Button", name, CraftBarFrame, "CraftBarButtonTemplate");
        
        button:RegisterForClicks("AnyUp");
        button.rangeTimer = -1;
	
        button:RegisterEvent("PLAYER_TARGET_CHANGED");
        button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

    	button:SetAttribute("type", "spell");
		button:SetAttribute("spell", spellname);
		button:SetAttribute("id", spellid);
		
		if (LBF) then
		    buttongroup:AddButton(button);
		end
			
		local icon = _G[button:GetName().."Icon"];
		icon:SetTexture(texture);
		
		craftbar_buttons[name] = button;
 	end
 	
 	initialized = true;
end

function CraftBar_initCraftFrame()	  

    if (not initialized) then    
        CraftBar_initButtons();	
    end 

    local count = 0;

	for name, spellid in pairs(craftbar_professions) do 	    
 	    local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	    local usable, nomana = IsUsableSpell(spellname);
 	    texture = GetSpellTexture(spellname);
 	    
 	    local button = craftbar_buttons[name];

 	    if (usable == 1) then
 	    
            button:ClearAllPoints();
            button:SetPoint("LEFT", (6 + 38 * count), 0);
			
            local icon = _G[button:GetName().."Icon"];
            icon:SetTexture(texture);
 	    
 	        button:Show();
 	        count = count + 1;
 	    else
 	        button:Hide();
 	    end
 	end
 	
    CraftBarFrame:SetWidth(10 + 38 * count);
    
    return count;
end     

function CraftBar_SnapDown()

	if (InCombatLockdown()) then
	    UIErrorsFrame:AddMessage(ERR_NOT_IN_COMBAT, 1.0, 0.0, 0.0, 1, 10);
	end	

	local frame = getglobal("CraftBarFrame");
	
	local count = CraftBar_initCraftFrame();
	
	if (count < 1) then
	    frame:Hide();
	    return;
	end
	
	def_posx = frame:GetLeft();
	def_posy = frame:GetBottom();
	
	local x, y = GetCursorPosition();
	local scale = UIParent:GetEffectiveScale();
	
	x = x / scale;	
	y = y / scale;
	
	local frame = getglobal("CraftBarFrame");
	frame:ClearAllPoints();
	frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
	
	frame:Show();
end

function CraftBar_SnapUp()
	local frame = getglobal("CraftBarFrame");
	frame:ClearAllPoints();
	frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", def_posx, def_posy);
	
    if (not InCombatLockdown()) then
	    frame:Hide();
	end
end

function CraftBar_MouseOver(button)
	CraftBar_SetTooltip(button);
end

function CraftBar_MouseOut(button)
	GameTooltip:Hide();
	selectedButton = nil;
end

function CraftBar_SetTooltip(button)

    local spellid = button:GetAttribute("id");
    
    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    
    GameTooltip:SetSpellByID(spellid);
        
    GameTooltip:AppendText("");
end